﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Input;

namespace LiquidacionGastos.Framework
{
    public class StateCommand<TState> 
    {
        private TState _CurrentStateId;
        public TState CurrentStateId 
        {
            get { return _CurrentStateId; }
            set 
            {
                this.CheckStateId(value);

                if (!object.Equals(_CurrentStateId, default(TState)))
                    this.CheckTransition(_CurrentStateId, value);

                this.PreviousStateId = _CurrentStateId;
                _CurrentStateId = value;

                ExecuteState(value);            }
        }

        public TState PreviousStateId { get; private set; }

        IDictionary<TState, Action<StateCommand<TState>>> States { get; set; }
        IDictionary<TState, List<TState>> TransitionsTo { get; set; }

        public StateCommand()
        {
            this._CurrentStateId = default(TState);
            this.PreviousStateId = default(TState);
            this.States = new Dictionary<TState, Action<StateCommand<TState>>>();
            this.TransitionsTo = new Dictionary<TState, List<TState>>();
        }

        public bool IsValidState(TState stateId)
        {
            return this.States.ContainsKey(stateId);
        }

        private void CheckStateId(TState stateId)
        {
            if (!this.IsValidState(stateId))
                throw new ArgumentOutOfRangeException("stateId", string.Format("State {0} not recognized", stateId.ToString()));
        }

        public bool IsValidTransition(TState nextStateId)
        {
            return this.IsValidTransition(this.CurrentStateId, nextStateId);
        }

        public bool IsValidTransition(TState currentStateId, TState nextStateId)
        {
            if (!this.TransitionsTo.ContainsKey(currentStateId))
                return false;

            if (!this.TransitionsTo[currentStateId].Contains(nextStateId))
                return false;

            return true;
        }

        private void CheckTransition(TState currentStateId, TState nextStateId)
        {
            if (!this.TransitionsTo.ContainsKey(currentStateId))
                throw new ArgumentException(string.Format("Transition could be executed because the state '{0}' is not defined as starting state for the transition.", currentStateId.ToString()), "currentStateId");

            if (!this.TransitionsTo[currentStateId].Contains(nextStateId))
                throw new ArgumentException(string.Format("Transition could be executed because the state '{0}' is not defined as ending state for the transition.", nextStateId.ToString()), "nextStateId");
        }

        private void ExecuteState(TState stateId)
        {
            this.States[stateId](this);
        }

        public void AddState(TState stateId, Action<StateCommand<TState>> execute, params TState[] validStateIdsTo)
        {
            if (this.States.ContainsKey(stateId))
                throw new ArgumentException(string.Format("State {0} can not be added because it already exits.", stateId.ToString()), "stateId");

            this.States.Add(stateId, execute);

            foreach (TState stateIdTo in validStateIdsTo)
                this.AddTransition(stateId, stateIdTo);    
        }

        private void AddTransition(TState stateId, TState stateIdTo)
        {
            if (!this.TransitionsTo.ContainsKey(stateId))
                this.TransitionsTo.Add(stateId, new List<TState>());

            IList<TState> stateIdToList = this.TransitionsTo[stateId];

            if (!stateIdToList.Contains(stateIdTo))
                stateIdToList.Add(stateIdTo);
        }
    }
}
