﻿using LiquidacionGastos.Framework;
using LiquidacionGastos.Models;
using System;
using System.Collections.Generic;
using System.Data.Entity;
using System.Data.Entity.Infrastructure;
using System.Linq;
using System.Linq.Expressions;
using System.Text;
using System.Threading.Tasks;

namespace LiquidacionGastos.Repositories
{
    public class EntityFrameworkRepository<TEntity> 
        where TEntity  : class
    {
        private readonly LGContext _Context;
        protected LGContext Context { get { return _Context; } }

        protected EntityFrameworkRepository(LGContext context)
        {
            if (context == null)
                throw new ArgumentNullException("context");

            _Context = context;
        }

        //protected EntityFrameworkRepository()
        //{
        //    _Context = ServiceLocator.Resolve<LGContext>();
        //}
        
        protected virtual TEntity Create()
        {
            return null;
        }

        protected virtual TEntity Get(Expression<Func<TEntity, bool>> predicate)
        {
            if (predicate == null)
                throw new ArgumentNullException("predicate");

            return Context.Set<TEntity>().FirstOrDefault(predicate);
        }

        protected virtual IQueryable<TEntity> List(Expression<Func<TEntity, bool>> predicate)
        {
            if (predicate == null)
                throw new ArgumentNullException("predicate");

            return Context.Set<TEntity>().Where(predicate);
        }

        protected virtual void Load(TEntity entity, string navigationProperty)
        {
            if (entity == null)
                throw new ArgumentNullException("entity");

            if (string.IsNullOrEmpty(navigationProperty))
                throw new ArgumentNullException("navigationProperty");

            Context.Entry<TEntity>(entity).Reference(navigationProperty).Load();            
        }

        protected virtual void Load<TProperty>(TEntity entity, Expression<Func<TEntity, TProperty>> navigationProperty) where TProperty : class
        {
            if (entity == null)
                throw new ArgumentNullException("entity");

            if (navigationProperty == null)
                throw new ArgumentNullException("navigationProperty");

            Context.Entry<TEntity>(entity).Reference(navigationProperty).Load();
        }

        protected virtual void LoadCollection(TEntity entity, string collectionProperty)
        {
            if (entity == null)
                throw new ArgumentNullException("entity");

            if (string.IsNullOrEmpty(collectionProperty))
                throw new ArgumentNullException("navigationProperty");

            Context.Entry<TEntity>(entity).Collection(collectionProperty).Load();
        }

        protected virtual void LoadCollection<TEntity, TElement>(TEntity entity, Expression<Func<TEntity, ICollection<TElement>>> navigationProperty) where TEntity : class where TElement : class
        {
            if (entity == null)
                throw new ArgumentNullException("entity");

            if (navigationProperty == null)
                throw new ArgumentNullException("navigationProperty");

            Context.Entry<TEntity>(entity).Collection<TElement>(navigationProperty).Load();
        }

        protected virtual void Insert(TEntity entity)
        {
            if (entity == null)
                throw new ArgumentNullException("entity");

#if DEBUG
            var v = Context.Entry<TEntity>(entity);
#endif

            if (Context.Entry<TEntity>(entity).State == EntityState.Detached)
                Context.Set<TEntity>().Attach(entity);

            Context.Set<TEntity>().Add(entity);
            OnAdd(entity);
        }

        protected virtual void Update(TEntity entity)
        {
            if (entity == null) 
                throw new ArgumentNullException("entity");

#if DEBUG
            var v = Context.Entry<TEntity>(entity);
#endif
            if (Context.Entry<TEntity>(entity).State == EntityState.Detached)
                Context.Set<TEntity>().Attach(entity);

            Context.Entry(entity).State = EntityState.Modified;
            OnUpdate(entity);
        }

        protected virtual void Delete(TEntity entity)
        {
            if (entity == null) 
                throw new ArgumentNullException("entity");

#if DEBUG
            var v = Context.Entry<TEntity>(entity);
#endif
            if (Context.Entry<TEntity>(entity).State == EntityState.Detached)
                Context.Set<TEntity>().Attach(entity);

            Context.Set<TEntity>().Remove(entity);
            OnDelete(entity);
        }

        protected virtual void Refresh(TEntity entity)
        {
#if DEBUG
            var v = Context.Entry<TEntity>(entity);
#endif
            if (Context.Entry<TEntity>(entity).State == EntityState.Detached)
                Context.Set<TEntity>().Attach(entity);

            Context.Entry<TEntity>(entity).Reload();
        }

        public void Attach(TEntity entity)
        {
            Context.Set<TEntity>().Attach(entity);
        }

        public void Detach(TEntity entity)
        {
            ((IObjectContextAdapter)Context).ObjectContext.Detach(entity);
        }

        protected virtual void OnAdd(TEntity entity)
        {
        }

        protected virtual void OnUpdate(TEntity entity)
        {
        }

        protected virtual void OnDelete(TEntity entity)
        {
        }

        public void LoadProperty(TEntity entity, string navigationProperty)
        {
            Load(entity, navigationProperty);
        }

        public void LoadProperty<TProperty>(TEntity entity, Expression<Func<TEntity, TProperty>> navigationProperty) where TProperty : class
        {
            this.Load(entity, navigationProperty);
        }

        public void LoadCollectionProperty(TEntity entity, string navigationProperty)
        {
            this.LoadCollection(entity, navigationProperty);
        }

        public void LoadCollectionProperty<TEntity, TElement>(TEntity entity, Expression<Func<TEntity, ICollection<TElement>>> navigationProperty) where TEntity : class where TElement : class
        {
            this.LoadCollection(entity, navigationProperty);
        }
    }
}
