﻿using LiquidacionGastos.Models;
using LiquidacionGastos.Repositories;
using System;
using System.Collections.Generic;
using System.Data.Entity;
using System.Linq;
using System.Linq.Expressions;
using System.Text;
using System.Threading.Tasks;

namespace LiquidacionGastos.UnitsOfWork
{
    public class LiquidacionUnitOfWork : EntityFrameworkTrackedUnitOfWork
    {
        private LiquidacionRepository LiquidacionRepository { get; set; }
        private UsuarioRepository UsuarioRepository { get; set; }
        private DepartamentoRepository DepartamentoRepository { get; set; }
        private CambioEstadoLiquidacionRepository CambioEstadoRepository { get; set; }
        private NotificacionRepository NotificacionRepository { get; set; }
        private GastoRepository GastoRepository { get; set; }

        public LiquidacionUnitOfWork() 
        {
        }

        public LiquidacionUnitOfWork(LGContext context) : base(context) 
        {            
        }

        public override void OnTrackedChanged()
        {
            this.LiquidacionRepository.IsTrackingEnabled = this.IsTrackingEnabled;
            this.CambioEstadoRepository.IsTrackingEnabled = this.IsTrackingEnabled;
            this.NotificacionRepository.IsTrackingEnabled = this.IsTrackingEnabled;
            this.GastoRepository.IsTrackingEnabled = this.IsTrackingEnabled;
        }

        protected override void OnInitializeRepositories()
        {
            this.LiquidacionRepository = new LiquidacionRepository(this.Context);
            this.CambioEstadoRepository = new CambioEstadoLiquidacionRepository(this.Context);
            this.NotificacionRepository = new NotificacionRepository(this.Context);
            this.GastoRepository = new GastoRepository(this.Context);
            this.UsuarioRepository = new UsuarioRepository(this.Context);
            this.DepartamentoRepository = new DepartamentoRepository(this.Context);
        }

        #region [Gestión de liquidaciones]

        ///// <summary>
        ///// Crear un nuevo objeto de tipo Liquidacion.
        ///// </summary>
        ///// <returns>El nuevo objeto de tipo Usuario</returns>
        public Liquidacion GetNewLiquidacion(Usuario usuario)
        {
            if (usuario == null)
                throw new ArgumentNullException("usuario");

            Liquidacion liquidacion = this.LiquidacionRepository.CreateLiquidacion(usuario);

            // configuración por defecto.
            liquidacion.Fecha = DateTime.Parse(DateTime.Today.ToString("yyyy/MM/01"));
            liquidacion.Usuario = usuario;
            //liquidacion.Calendario = usuario.Calendario
            this.ChangeEstadoLiquidacion(liquidacion, EstadoLiquidacionType.Abierta);
            
            return liquidacion;
        }

        public CambioEstadoLiquidacion ChangeEstadoLiquidacion(Liquidacion liquidacion, EstadoLiquidacionType estadoNuevo)
        {
            return this.ChangeEstadoLiquidacion(liquidacion, estadoNuevo, string.Empty);            
        }

        public CambioEstadoLiquidacion ChangeEstadoLiquidacion(Liquidacion liquidacion, EstadoLiquidacionType estadoNuevo, string motivoRechazo)
        {
            if (liquidacion == null)
                throw new ArgumentNullException("liquidacion");

            EstadoLiquidacion estadoLiquidacionActual = liquidacion.Estado;
            EstadoLiquidacion estadoLiquidacionNuevo = MasterTypeFactory.GetEstadoLiquidacion(estadoNuevo);

            liquidacion.Estado = estadoLiquidacionNuevo;

            CambioEstadoLiquidacion cambioEstado = this.CambioEstadoRepository.CreateCambioEstadoLiquidacion(liquidacion, estadoLiquidacionActual, liquidacion.Estado);            
            cambioEstado.MotivoRechazo = motivoRechazo;
            liquidacion.CambiosEstado.Add(cambioEstado);

            Notificacion notificacion = this.NotificacionRepository.CreateNotificacion(cambioEstado);
            cambioEstado.Notificaciones.Add(notificacion);

            return cambioEstado;
        }

        public Liquidacion GetLiquidacion(Guid idLiquidacion)
        {
            if (idLiquidacion == null)
                throw new ArgumentNullException("idLiquidacion");

            return this.LiquidacionRepository.GetLiquidacion(idLiquidacion);
        }

        public void InsertLiquidacion(Liquidacion liquidacion)
        {
            if (liquidacion == null)
                throw new ArgumentNullException("liquidacion");

            this.LiquidacionRepository.InsertLiquidacion(liquidacion);
        }

        public void UpdateLiquidacion(Liquidacion liquidacion)
        {
            if (liquidacion == null)
                throw new ArgumentNullException("liquidacion");

            this.LiquidacionRepository.UpdateLiquidacion(liquidacion);
        }

        public void UpdateImporte(Liquidacion liquidacion)
        {
            this.LiquidacionRepository.UpdateImporte(liquidacion);
        }

        public void DeleteLiquidacion(Liquidacion liquidacion)
        {
            if (liquidacion == null)
                throw new ArgumentNullException("liquidacion");

            this.LiquidacionRepository.DeleteLiquidacion(liquidacion);
        }

        public bool ValidateLiquidacion(Liquidacion liquidacion)
        {
            return true;
        }

        public void CheckLiquidacionAbierta(Usuario usuario)
        {
            // Si no existe ninguna liquidación abierta. 
            if (this.LiquidacionRepository.ListLiquidaciones(liquidacion => liquidacion.IdUsuario == usuario.IdUsuario &&
                                                                 liquidacion.IdEstado == (int)EstadoLiquidacionType.Abierta).Count() == 0)
            {
                Liquidacion liquidacion = this.GetNewLiquidacion(usuario);
                this.InsertLiquidacion(liquidacion);
            }
        }

        public ICollection<Liquidacion> ListLiquidacionesByUsuario(Usuario usuario, params EstadoLiquidacionType[] estadosLiquidacion)
        {
            // Copiar los estados requeridos en una lista para acelerar la búsqueda.
            IList<int> estadosRequeridos = new List<int>();
            foreach(EstadoLiquidacionType estado in estadosLiquidacion)
                estadosRequeridos.Add((int)estado);
            
            // Obtener lista la de liquidaciones.
            return this.LiquidacionRepository.ListLiquidaciones(liquidacion => liquidacion.IdUsuario == usuario.IdUsuario &&
                                                                estadosRequeridos.Contains(liquidacion.IdEstado))
                                                                .ToList();
        }

        /// <summary>
        /// Devuelve la lista de liquidaciones en las que el usuario es autorizador.
        /// </summary>
        /// <param name="autorizador"></param>
        /// <returns>La colección de </returns>
        public ICollection<Liquidacion> ListLiquidacionesByAutorizador(Usuario autorizador, params EstadoLiquidacionType[] estadosLiquidacion)
        {
            // Crea una lista de los identificadores de los usuarios supervisados.
            List<System.Guid> usuariosSupervisados = new List<System.Guid>();

            // Carga la lista con los usuarios de los departarmentos supervisados por el autorizador.
            this.UsuarioRepository.LoadCollectionProperty(autorizador, t => t.DepartamentosSupervisados);            
            foreach (Departamento departamento in autorizador.DepartamentosSupervisados)
            {
                this.DepartamentoRepository.LoadCollectionProperty(departamento, t => t.Usuarios);
                foreach (Usuario usuario in departamento.Usuarios)
                    if (!usuariosSupervisados.Contains(usuario.IdUsuario))
                        usuariosSupervisados.Add(usuario.IdUsuario);
            }

            // Copiar los estados requeridos en una lista para acelerar la búsqueda.
            IList<int> estadosRequeridos = new List<int>();
            foreach(EstadoLiquidacionType estado in estadosLiquidacion)
                estadosRequeridos.Add((int)estado);

            // Obtener lista la de liquidaciones de todos los usuarios supervisados.
            return this.LiquidacionRepository.ListLiquidaciones(liquidacion => usuariosSupervisados.Contains(liquidacion.IdUsuario) && 
                                                                               estadosRequeridos.Contains(liquidacion.IdEstado))
                                                                               .ToList(); 
        }

        #endregion [Gestión de liquidaciones]

        #region [Gestión de gastos]

        public Gasto GetNewGasto(Liquidacion liquidacion)
        {
            Gasto gasto = this.GastoRepository.CreateGasto(liquidacion);

            // configuración por defecto.
            gasto.Fecha = DateTime.Today;            
            gasto.Tipo = MasterTypeFactory.GetTipoGasto(TipoGastoType.Otros);
            gasto.Descripcion = string.Empty;
            gasto.Importe = 0;
            //gasto.CambioMoneda = liquidacion.Usuario.CambioMoneda

            return gasto;
        }

        public Gasto GetGasto(Guid idGasto)
        {
            if (idGasto == null)
                throw new ArgumentNullException("idGasto");

            return this.GastoRepository.GetGasto(idGasto);
        }

        public void InsertGasto(Gasto gasto)
        {
            if (gasto == null)
                throw new ArgumentNullException("gasto");

            this.GastoRepository.InsertGasto(gasto);
        }

        public void UpdateGasto(Gasto gasto)
        {
            if (gasto == null)
                throw new ArgumentNullException("gasto");

            this.GastoRepository.UpdateGasto(gasto);
        }

        public void DeleteGasto(Gasto gasto)
        {
            if (gasto == null)
                throw new ArgumentNullException("gasto");

            this.GastoRepository.DeleteGasto(gasto);
        }

        public void RefreshGasto(Gasto gasto)
        {
            if (gasto == null)
                throw new ArgumentNullException("gasto");

            this.GastoRepository.RefreshGasto(gasto);
        }

        public ICollection<Gasto> ListGastos()
        {
            Expression<Func<Gasto, bool>> filterPredicate = (u => true);

            return this.GastoRepository.ListGastos(filterPredicate).ToList();
        }

        public ICollection<Gasto> ListGastosByLiquidacion(Liquidacion liquidacion)
        {
            if (liquidacion == null)
                throw new ArgumentNullException("liquidacion");

            Expression<Func<Gasto, bool>> filterPredicate = null;
            filterPredicate = (t => t.IdLiquidacion == liquidacion.IdLiquidacion);

            return this.GastoRepository.ListGastos(filterPredicate)
                                       .Include(t => t.Liquidacion)
                                       .Include(t => t.Tipo)
                                       .Include(t => t.CambioMoneda)
                                       .Include(t => t.CambioMoneda.Moneda)
                                       .ToList();
        }

        #endregion [Gestión de gastos]

        #region [Gestión de tipos de gastos]

        public ICollection<TipoGasto> ListTiposGasto()
        {
            return MasterTypeFactory.ListTiposGasto();
        }

        public ICollection<CambioMoneda> ListCambiosMonedaMostRecent()
        {
            return MasterTypeFactory.ListCambiosMonedaMostRecent();
        }

        #endregion [Gestión de tipos de gastos]
    }
}
