﻿using LiquidacionGastos.Models;
using LiquidacionGastos.Repositories;
using System;
using System.Collections.Generic;
using System.Data.Entity;
using System.Linq;
using System.Linq.Expressions;
using System.Text;
using System.Threading.Tasks;

namespace LiquidacionGastos.UnitsOfWork
{
    public class OrganizacionUnitOfWork : EntityFrameworkUnitOfWork
    {
        private UsuarioRepository UsuarioRepository { get; set; }
        private DepartamentoRepository DepartamentoRepository { get; set; }
        private CalendarioRepository CalendarioRepository { get; set; }

        public OrganizacionUnitOfWork() 
        {
        }

        public OrganizacionUnitOfWork(LGContext context) : base(context) 
        {            
        }

        protected override void OnInitializeRepositories()
        {
            this.UsuarioRepository = new UsuarioRepository(this.Context);
            this.DepartamentoRepository = new DepartamentoRepository(this.Context);
            this.CalendarioRepository = new CalendarioRepository(this.Context);
        }

        #region [Gestión de usuarios]

        /// <summary>
        /// Crear un nuevo objeto de tipo Usuario.
        /// </summary>
        /// <returns>El nuevo objeto de tipo Usuario</returns>
        public Usuario GetNewUsuario()
        {
            Usuario usuario = this.UsuarioRepository.CreateUsuario();

            // configuración por defecto.
            usuario.EstaActivadoLogin = true;

            return usuario;
        }

        /// <summary>
        /// Devuelve un usuario determinado por el valor de su propiedad identificador.
        /// No confundir la propiedad identificador con IdUsuario.
        /// </summary>
        /// <param name="identificador"></param>
        public Usuario GetUsuarioByIdentificador(string identificador)
        {
            if (string.IsNullOrEmpty(identificador))
                throw new ArgumentNullException("identificador");

            return this.UsuarioRepository.GetUsuarioByIdentificador(identificador);
        }

        public Usuario GetUsuario(Guid idUsuario)
        {
            if (idUsuario == null)
                throw new ArgumentNullException("idUsuario");

            return this.UsuarioRepository.GetUsuario(idUsuario);
        }

        public void InsertUsuario(Usuario usuario)
        {
            if (usuario == null)
                throw new ArgumentNullException("usuario");

            this.UsuarioRepository.InsertUsuario(usuario);
        }

        public void UpdateUsuario(Usuario usuario)
        {
            if (usuario == null)
                throw new ArgumentNullException("usuario");

            this.UsuarioRepository.UpdateUsuario(usuario);
        }

        public void DeleteUsuario(Usuario usuario)
        {
            if (usuario == null)
                throw new ArgumentNullException("usuario");

            this.UsuarioRepository.DeleteUsuario(usuario);
        }

        public void RefreshUsuario(Usuario usuario)
        {
            if (usuario == null)
                throw new ArgumentNullException("usuario");

            this.UsuarioRepository.RefreshUsuario(usuario);
        }

        public ICollection<Usuario> ListUsuarios()
        {
            Expression<Func<Usuario, bool>> filterPredicate = (u => true);

            return this.UsuarioRepository.ListUsuarios(filterPredicate).ToList();
        }

        public ICollection<Usuario> ListUsuariosByNombreUsuario(string nombreUsuario)
        {
            Expression<Func<Usuario, bool>> filterPredicate = (u => u.Nombre.Contains(nombreUsuario));

            return this.UsuarioRepository.ListUsuarios(filterPredicate).ToList();
        }

        public ICollection<Usuario> ListUsuariosByNombreDepartamento(string nombreDepartamento)
        {
            Expression<Func<Usuario, bool>> filterPredicate = null;
            if (nombreDepartamento == "(!)")
                filterPredicate = (u => u.Departamento == null);
            else
                filterPredicate = (u => u.Departamento.Nombre.Contains(nombreDepartamento));

            return this.UsuarioRepository.ListUsuarios(filterPredicate).ToList();
        }

        public void LoadDepartamento(Usuario usuario)
        {
            this.UsuarioRepository.LoadDepartamento(usuario);
        }

        public void LoadSubdepartamentos(Usuario usuario)
        {
            this.UsuarioRepository.LoadSubdepartamentos(usuario);
        }

        #endregion [Gestión de usuarios]

        #region [Gestión de departamentos]

        public Departamento GetNewDepartamento()
        {
            Departamento departamento = this.DepartamentoRepository.CreateDepartamento();

            // configuración por defecto.
            // (no aplica)

            return departamento;
        }

        public Departamento GetDepartamento(Guid idDepartamento)
        {
            if (idDepartamento == null)
                throw new ArgumentNullException("idDepartamento");

            return this.DepartamentoRepository.GetDepartamento(idDepartamento);
        }

        public void InsertDepartamento(Departamento departamento)
        {
            if (departamento == null)
                throw new ArgumentNullException("departamento");

            this.DepartamentoRepository.InsertDepartamento(departamento);
        }

        public void UpdateDepartamento(Departamento departamento)
        {
            if (departamento == null)
                throw new ArgumentNullException("departamento");

            this.DepartamentoRepository.UpdateDepartamento(departamento);
        }

        public void DeleteDepartamento(Departamento departamento)
        {
            if (departamento == null)
                throw new ArgumentNullException("departamento");

            this.DepartamentoRepository.DeleteDepartamento(departamento);
        }

        public void RefreshDepartamento(Departamento departamento)
        {
            if (departamento == null)
                throw new ArgumentNullException("departamento");

            this.DepartamentoRepository.RefreshDepartamento(departamento);
        }

        public ICollection<Departamento> ListDepartamentos()
        {
            Expression<Func<Departamento, bool>> filterPredicate = (d => true);

            return this.DepartamentoRepository.ListDepartamentos(filterPredicate).ToList();
        }

        public ICollection<Departamento> ListDepartamentosByNombreDepartamento(string nombreDepartamento)
        {
            Expression<Func<Departamento, bool>> filterPredicate = (d => d.Nombre.Contains(nombreDepartamento));

            return this.DepartamentoRepository.ListDepartamentos(filterPredicate).ToList();
        }

        public ICollection<Departamento> ListDepartamentosBySuperdepartamento(Departamento superdepartamento)
        {
            Expression<Func<Departamento, bool>> filterPredicate = (d => d.Superdepartamento == superdepartamento);

            return this.DepartamentoRepository.ListDepartamentos(filterPredicate).ToList();
        }

        #endregion [Gestión de departamentos]

        #region [Gestión de calendarios]

        public Calendario GetNewCalendario()
        {
            Calendario calendario = this.CalendarioRepository.CreateCalendario();

            // configuración por defecto.
            calendario.Nombre = "(calendario sin nombre)";
            calendario.DiasFestivos = new byte[50]; // 4+46*8 = 368 días > 366 días

            calendario.DiasFestivos[1] = 1; 

            return calendario;
        }

        public void InsertCalendario(Calendario calendario)
        {
            if (calendario == null)
                throw new ArgumentNullException("calendario");

            this.CalendarioRepository.InsertCalendario(calendario);
        }

        public void UpdateCalendario(Calendario calendario)
        {
            if (calendario == null)
                throw new ArgumentNullException("calendario");

            this.CalendarioRepository.UpdateCalendario(calendario);
        }

        public void DeleteCalendario(Calendario calendario)
        {
            if (calendario == null)
                throw new ArgumentNullException("calendario");

            this.CalendarioRepository.DeleteCalendario(calendario);
        }

        public void RefreshCalendario(Calendario calendario)
        {
            if (calendario == null)
                throw new ArgumentNullException("calendario");

            this.CalendarioRepository.RefreshCalendario(calendario);
        }

        public ICollection<Calendario> ListCalendarios()
        {
            Expression<Func<Calendario, bool>> filterPredicate = (c => true);

            return this.CalendarioRepository.ListCalendarios(filterPredicate).ToList();
        }

        public ICollection<Calendario> ListCalendariosByNombreCalendario(string nombreCalendario)
        {
            Expression<Func<Calendario, bool>> filterPredicate = (c => c.Nombre.Contains(nombreCalendario));

            return this.CalendarioRepository.ListCalendarios(filterPredicate).ToList();
        }

        public Calendario DuplicateCalendario(Calendario calendario)
        {
            Calendario newCalendario = this.GetNewCalendario();

            newCalendario.Nombre = "Copia de " + calendario.Nombre;
            newCalendario.DiasFestivos = calendario.DiasFestivos;

            return newCalendario;
        }

        public void ResetCalendario(Calendario calendario)
        {
            calendario.DiasFestivos = new byte[50];
        }

        #endregion [Gestión de calendarios]

    }

}
