﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Collections.Specialized;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;

namespace LiquidacionGastos.Framework
{
    public class CalendarHelper
    {
        #region Attributes

        private static readonly Dictionary<Calendar, ObservableCollection<DateTime>> _BlackoutDatesCalendars = new Dictionary<Calendar, ObservableCollection<DateTime>>();

        private static readonly Dictionary<Calendar, ObservableCollection<DateTime>> _SelectedDatesCalendars = new Dictionary<Calendar, ObservableCollection<DateTime>>();

        #endregion

        #region Dependency properties

        public static DependencyProperty BlackoutDatesProperty = DependencyProperty.RegisterAttached("BlackoutDates", typeof(ObservableCollection<DateTime>), typeof(CalendarHelper), new PropertyMetadata(null, OnRegisterCalendarBlackoutDatesPropertyBindingChanged));

        public static void SetBlackoutDates(DependencyObject d, ObservableCollection<DateTime> value)
        {
            d.SetValue(BlackoutDatesProperty, value);
        }

        public static ObservableCollection<DateTime> GetBlackoutDates(DependencyObject d)
        {
            return (ObservableCollection<DateTime>)d.GetValue(BlackoutDatesProperty);
        }

        public static DependencyProperty SelectedDatesProperty = DependencyProperty.RegisterAttached("SelectedDates", typeof(ObservableCollection<DateTime>), typeof(CalendarHelper), new PropertyMetadata(null, OnRegisterCalendarSelectedDatesPropertyBindingChanged));

        public static void SetSelectedDates(DependencyObject d, ObservableCollection<DateTime> value)
        {
            d.SetValue(SelectedDatesProperty, value);
        }

        public static ObservableCollection<DateTime> GetSelectedDates(DependencyObject d)
        {
            return (ObservableCollection<DateTime>)d.GetValue(SelectedDatesProperty);
        }

        #endregion

        #region Register property actions

        private static void OnRegisterCalendarBlackoutDatesPropertyBindingChanged(DependencyObject sender, DependencyPropertyChangedEventArgs e)
        {
            Calendar calendar = sender as Calendar;
            if (calendar == null)
                return;

            ObservableCollection<DateTime> oldBlackoutDatesCollection = e.OldValue as ObservableCollection<DateTime>;
            if (oldBlackoutDatesCollection != null)
            {
                oldBlackoutDatesCollection.CollectionChanged -= OnCollectionBlackoutDatesCollectionChanged;

                if (_BlackoutDatesCalendars.Keys.Contains(calendar))
                    _BlackoutDatesCalendars.Remove(calendar);
            }

            ObservableCollection<DateTime> newBlackoutDatesCollection = e.NewValue as ObservableCollection<DateTime>;
            if (newBlackoutDatesCollection != null)
            {
                if (!_BlackoutDatesCalendars.Keys.Contains(calendar))
                    _BlackoutDatesCalendars.Add(calendar, newBlackoutDatesCollection);
           
                calendar.BlackoutDates.Clear();
                foreach (DateTime date in newBlackoutDatesCollection)
                    calendar.BlackoutDates.Add(new CalendarDateRange(date));

                newBlackoutDatesCollection.CollectionChanged += OnCollectionBlackoutDatesCollectionChanged;
            }
            
        }

        private static void OnRegisterCalendarSelectedDatesPropertyBindingChanged(DependencyObject sender, DependencyPropertyChangedEventArgs e)
        {
            Calendar calendar = sender as Calendar;
            if (calendar == null)
                return;

            ObservableCollection<DateTime> oldSelectedDatesCollection = e.OldValue as ObservableCollection<DateTime>;
            if (oldSelectedDatesCollection != null)
            {
                oldSelectedDatesCollection.CollectionChanged -= OnCollectionSelectedDatesCollectionChanged;
                calendar.SelectedDatesChanged -= OnCalendarSelectedDatesCollectionChanged;

                if (_SelectedDatesCalendars.Keys.Contains(calendar))
                    _SelectedDatesCalendars.Remove(calendar);
            }

            ObservableCollection<DateTime> newSelectedDatesCollection = e.NewValue as ObservableCollection<DateTime>;
            if (newSelectedDatesCollection != null)
            {
                if (!_SelectedDatesCalendars.Keys.Contains(calendar))
                    _SelectedDatesCalendars.Add(calendar, newSelectedDatesCollection);

                calendar.SelectedDates.Clear();
                foreach (DateTime date in newSelectedDatesCollection)
                    calendar.SelectedDates.Add(date);

                newSelectedDatesCollection.CollectionChanged += OnCollectionSelectedDatesCollectionChanged;
                calendar.SelectedDatesChanged += OnCalendarSelectedDatesCollectionChanged;
            }

        }

        #endregion

        #region Changing property value actions

        private static void OnCollectionBlackoutDatesCollectionChanged(object sender, System.Collections.Specialized.NotifyCollectionChangedEventArgs e)
        {
            ObservableCollection<DateTime> blackoutDates = sender as ObservableCollection<DateTime>;

            Calendar calendar = _BlackoutDatesCalendars.First(kvp => kvp.Value == blackoutDates).Key;

            ObservableCollection<DateTime> selectedDates = _SelectedDatesCalendars.First(kvp => kvp.Key == calendar).Value;

            if (e.Action == NotifyCollectionChangedAction.Add)
            {
                foreach (DateTime date in e.NewItems)
                {
                    if (selectedDates.Contains(date))
                        selectedDates.Remove(date);

                    calendar.BlackoutDates.Add(new CalendarDateRange(date));
                }
            }
            else if (e.Action == NotifyCollectionChangedAction.Remove) {
                foreach (DateTime date in e.OldItems)
                    if (calendar.BlackoutDates.Contains(date))
                        calendar.BlackoutDates.Remove(calendar.BlackoutDates.First(d => d.Start.Date == date.Date));
            }
            else if (e.Action == NotifyCollectionChangedAction.Reset){
                calendar.BlackoutDates.Clear();
            }
        }

        private static void OnCalendarSelectedDatesCollectionChanged(object sender, SelectionChangedEventArgs e)
        {
            Calendar calendar = sender as Calendar;

            if (calendar == null)
                return;

            ObservableCollection<DateTime> selectedDates = _SelectedDatesCalendars.First(kvp => kvp.Key == calendar).Value;

            selectedDates.CollectionChanged -= OnCollectionSelectedDatesCollectionChanged;

            if (e.AddedItems.Count > 0)
            {              
                foreach (DateTime date in e.AddedItems)
                {                    
                    selectedDates.Add(date);
                }
            }

            if (e.RemovedItems.Count > 0)
            {
                foreach (DateTime date in e.RemovedItems)
                {
                    if (selectedDates.Contains(date))
                        selectedDates.Remove(date);
                }
            }

            selectedDates.CollectionChanged += OnCollectionSelectedDatesCollectionChanged;
        }

        private static void OnCollectionSelectedDatesCollectionChanged(object sender, System.Collections.Specialized.NotifyCollectionChangedEventArgs e)
        {

            ObservableCollection<DateTime> selectedDates = sender as ObservableCollection<DateTime>;

            Calendar calendar = _SelectedDatesCalendars.First( kvp => kvp.Value == selectedDates).Key;

            calendar.SelectedDatesChanged -= OnCalendarSelectedDatesCollectionChanged;

            if (e.Action == NotifyCollectionChangedAction.Add)
            {
                foreach (DateTime date in e.NewItems)
                {
                    calendar.SelectedDates.Add(date);
                }
            }

            if (e.Action == NotifyCollectionChangedAction.Remove)
            {
                foreach (DateTime date in e.OldItems)
                {
                    if (calendar.SelectedDates.Contains(date))
                        calendar.SelectedDates.Remove(date);
                }
            }

            if (e.Action == NotifyCollectionChangedAction.Reset)
            {
                calendar.SelectedDates.Clear();
            }

            calendar.SelectedDatesChanged += OnCalendarSelectedDatesCollectionChanged;
        }

        #endregion
    }
}
