﻿using LiquidacionGastos.Business;
using LiquidacionGastos.Framework;
using LiquidacionGastos.Models;
using LiquidacionGastos.Services;
using System;
using System.Collections.Generic;
//using System.Collections.ObjectModel;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;

namespace LiquidacionGastos.ViewModels
{
    public class DepartamentoListViewModel : ViewModel
    {
        #region [Subsistema básico]

        public RelayCommand CancelCommand { get; set; }

        public DepartamentoListViewModel(bool isSelectionMode)
        {
            this.ViewId = ViewName.DepartamentoListView;
            this.DepartamentoManagement = new OrganizacionManagement();            
            this.DepartamentoList = new ObservableCollection<Departamento>();
            this.IsSelectionMode = isSelectionMode;

            if (this.IsSelectionMode)
            {
                this.ViewTitle = "Selección de departamento";
                this.SelectionButtonVisibility = Visibility.Visible;
            }
            else
            {
                this.ViewTitle = "Lista de departamentos";
                this.SelectionButtonVisibility = Visibility.Hidden;
            }

            // Cargar los filtros de selección de departamentos.
            this.FilterList.AddRange(this.FILTERS);

            // Seleccionar filtro sin efecto. Se cargan todos los departamentos.
            this.CurrentFilter = FILTER_SIN_FILTRO;

            this.CancelCommand = new RelayCommand(OnCancel);
            this.CreateDepartamentoCommand = new RelayCommand(OnCreateDepartamento);
            this.EditDepartamentoCommand = new RelayCommand(OnEditDepartamento, OnCanEditDepartamento);
            this.DeleteDepartamentoCommand = new RelayCommand(OnDeleteDepartamento, OnCanDeleteDepartamento);
            this.SelectDepartamentoCommand = new RelayCommand(OnSelectDepartamento, OnCanSelectDepartamento);
        }

        private void OnCancel(object param)
        {
            this.Stop(0);
        }

        #endregion [Subsistema básico]

        #region [Subsistema de mantenimiento]

        private Departamento _CurrentDepartamento;
        public Departamento CurrentDepartamento { get { return _CurrentDepartamento; } set { _CurrentDepartamento = value; OnPropertyChanged(); } }

        public ObservableCollection<Departamento> DepartamentoList { get; private set; }

        private OrganizacionManagement DepartamentoManagement { get; set; }

        public RelayCommand CreateDepartamentoCommand { get; private set; }
        public RelayCommand EditDepartamentoCommand { get; private set; }
        public RelayCommand DeleteDepartamentoCommand { get; private set; }

        private void OnCreateDepartamento(object param)
        {
            this.CurrentDepartamento = this.DepartamentoManagement.GetNewDepartamento();
            DepartamentoViewModel vm = new DepartamentoViewModel();
            vm.CurrentDepartamento = this.CurrentDepartamento;
            vm.Start(OnCreateDepartamentoCallback);
        }

        private void OnCreateDepartamentoCallback(ViewModel sender, ViewModelStoppedEventArgs e)
        {
            DepartamentoViewModel vm = sender as DepartamentoViewModel;
            if (e.Result != 1)
            {
                Debug.WriteLine("Datos del nuevo Departamento descartados.");
                return;
            }

            this.DepartamentoManagement.InsertDepartamento(vm.CurrentDepartamento);
            this.DepartamentoList.Add(vm.CurrentDepartamento);
            Debug.WriteLine("Datos del nuevo departamento actualizados en la base de datos");

            // Recargar la lista si ha cambiado el departamento al que pertenece el actual departamento.
            this.OnCurrentFilterChanged();
        }

        private bool OnCanEditDepartamento(object param)
        {
            return (this.CurrentDepartamento != null);
        }

        private void OnEditDepartamento(object param)
        {
            DepartamentoViewModel wm = new DepartamentoViewModel();
            wm.CurrentDepartamento = this.CurrentDepartamento;
            wm.Start(OnEditDepartamentoCallback);
        }

        private void OnEditDepartamentoCallback(ViewModel sender, ViewModelStoppedEventArgs e)
        {
            DepartamentoViewModel vm = sender as DepartamentoViewModel;
            if (e.Result != 1)
            {
                this.DepartamentoManagement.RefreshDepartamento(vm.CurrentDepartamento);
                Debug.WriteLine("Datos del departamento recargados desde el contexto de la base de datos");
                return;
            }
                        
            this.DepartamentoManagement.UpdateDepartamento(vm.CurrentDepartamento);
            Debug.WriteLine("Datos del departamento actualizados en la base de datos");

            // Recargar la lista si ha cambiado el departamento al que pertenece el actual departamento.
            this.OnCurrentFilterChanged();
        }

        private bool OnCanDeleteDepartamento(object obj)
        {
            return (this.CurrentDepartamento != null);
        }

        private void OnDeleteDepartamento(object obj)
        {
            DeleteDepartamentoMessageViewModel vm = new DeleteDepartamentoMessageViewModel();
            vm.Start(OnDeleteDepartamentoCallback);
        }

        private void OnDeleteDepartamentoCallback(ViewModel sender, ViewModelStoppedEventArgs e)
        {
            if (e.Result != 1)
                return;

            // Obtener el departamento seleccionado actualmente.
            Departamento departamentoActual = this.CurrentDepartamento;

            // Eliminar el departamento de la lista. La propiedad this.CurrentDepartamento se convierte a null.
            this.DepartamentoList.Remove(this.CurrentDepartamento);

            // Eliminar el usuario del sistema.
            this.DepartamentoManagement.DeleteDepartamento(departamentoActual);
        }
        
        #endregion [Subsistema de mantenimiento]

        #region [Subsistema de filtrado]

        private const string FILTER_SIN_FILTRO = "(sin filtro)";        
        private const string FILTER_NOMBRE_DEPARTAMENTO = "nombre departamento";

        private readonly string[] FILTERS = new string[] { FILTER_SIN_FILTRO, FILTER_NOMBRE_DEPARTAMENTO };

        private string _CurrentFilter;
        public string CurrentFilter { get { return _CurrentFilter; } set { _CurrentFilter = value; OnPropertyChanged(); OnCurrentFilterChanged(); } }

        private ObservableCollection<string> _FilterList = new ObservableCollection<string>();
        public ObservableCollection<string> FilterList { get { return _FilterList; } set { _FilterList = value; } }

        private string _FilterText = string.Empty;
        public string FilterText { get { return _FilterText; } set { _FilterText = value; OnPropertyChanged(); OnCurrentFilterChanged(); } }

        private void OnCurrentFilterChanged()
        {
            string newFilter = this.CurrentFilter;
            string filterText = this.FilterText;

            //Debug.Assert(newFilter == null, "El el texto del filtro tiene valor null");
            //Debug.Assert(filterText == null, "El filtro seleccionado tiene valor null");

            if (newFilter == null || filterText == null)
                return;

            ICollection<Departamento> departamentos = null;

            switch (newFilter)
            {
                case FILTER_SIN_FILTRO:
                    departamentos = this.DepartamentoManagement.ListDepartamentosBySuperdepartamento(null);
                    break;

                case FILTER_NOMBRE_DEPARTAMENTO:
                    departamentos = this.DepartamentoManagement.ListDepartamentosByNombreDepartamento(filterText);
                    break;
            }

            this.DepartamentoList.ReplaceRange(departamentos);
        }

        #endregion [Subsistema de filtrado]
        
        #region [Subsistema de selección]

        public bool IsSelectionMode { get; private set; }

        public Visibility SelectionButtonVisibility { get; private set; }

        public RelayCommand SelectDepartamentoCommand { get; private set; }
        
        private bool OnCanSelectDepartamento(object obj)
        {
            return (this.CurrentDepartamento != null);
        }

        private void OnSelectDepartamento(object param)
        {
            this.Stop(1);
        }

        #endregion [Subsistema de selección]
    }
}
