﻿using LiquidacionGastos.Business;
using LiquidacionGastos.Framework;
using LiquidacionGastos.Models;
using LiquidacionGastos.Services;
using System;
using System.Collections.Generic;
//using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Security.Principal;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Data;

namespace LiquidacionGastos.ViewModels
{
    /// <summary>
    /// Vista que muestra la lista de liquidaciones del usuario ó las que debe autorizar el autorizador.
    /// Cuando el atributo IsAuthorizationMode es igual a false, muestra las del usuario, mientras que si
    /// es false, muestra las del autorizador.
    /// </summary>
    /// <seealso cref="IsAutorizationMode"/>
    public class LiquidacionListViewModel : ViewModel
    {
        #region [Subsistema básico]

        /// <summary>
        /// Indica a la vista si debe mostrar las liquidaciones del usuario que consulta ó las que debe
        /// autorizar el autorizador.
        /// </summary>
        public bool IsAuthorizationMode { get; set; }

        private Liquidacion _CurrentLiquidacion;
        public Liquidacion CurrentLiquidacion { get { return _CurrentLiquidacion; } set { _CurrentLiquidacion = value; OnPropertyChanged(); } }

        /// <summary>
        /// Lista de objetos liquidación que muestra la lista.
        /// </summary>
        public ObservableCollection<Liquidacion> LiquidacionList { get; private set; }

        public Usuario CurrentUsuario { get; set; }

        public RelayCommand CancelCommand { get; private set; }
        public RelayCommand EditLiquidacionCommand { get; private set; }

        private LiquidacionManagement LiquidacionManagement { get; set; }
        
        public LiquidacionListViewModel(bool isAuthorizationMode)
        {
            this.ViewId = ViewName.LiquidacionListView;
            this.IsAuthorizationMode = isAuthorizationMode;
            this.LiquidacionList = new ObservableCollection<Liquidacion>();
            this.LiquidacionManagement = new LiquidacionManagement();

            IAuthenticationService service = ServiceLocator.Resolve<IAuthenticationService>();
            this.CurrentUsuario = service.UsuarioPrincipal.Usuario;

            if (this.IsAuthorizationMode)
            {                
                this.ViewTitle = "Lista de autorizaciones de gastos";
            }
            else
            {
                this.ViewTitle = "Lista de liquidaciones de gastos";
            }

            // Cargar los filtros de selección de liquidaciones.
            this.FilterList.AddRange(this.FILTERS);

            // Seleccionar filtro sin efecto. Se cargan todas liquidaciones de los usuarios dependendientes.
            this.CurrentFilter = FILTER_SIN_FILTRO;

            this.CancelCommand = new RelayCommand(OnCancelCommand);
            this.EditLiquidacionCommand = new RelayCommand(OnEditLiquidacion, OnCanEditLiquidacion);
        }

        private void OnCancelCommand(object param)
        {
            this.Stop(0);
        }

        #endregion [Subsistema básico]

        #region [Subsistema de mantenimiento]

        private bool OnCanEditLiquidacion(object param)
        {
            return this.CurrentLiquidacion != null;
        }

        private void OnEditLiquidacion(object param)
        {
            LiquidacionViewModel vm = new LiquidacionViewModel(this.IsAuthorizationMode);
            vm.CurrentLiquidacion = this.CurrentLiquidacion;
            vm.Start();
        }

        //private void OnEditLiquidacionCallback(ViewModel sender, ViewModelStoppedEventArgs e)
        //{
        //    LiquidacionViewModel vm = sender as LiquidacionViewModel;
        //    if (e.Result != 1)
        //    {
        //    }
        //}

        #endregion [Subsistema de mantenimiento]

        #region [Subsistema de filtrado]

        private const string FILTER_SIN_FILTRO = "(sin filtro)";
        private const string FILTER_ESTADO_ABIERTA = "estado Abierta";
        private const string FILTER_ESTADO_ESPERANDO_AUTORIZACION = "estado Esperando autorización";
        private const string FILTER_ESTADO_RECHAZADA = "estado Rechazada";
        private const string FILTER_ESTADO_AUTORIZADA = "estado Autorizada";
        private const string FILTER_ESTADO_CERRADA = "estado Cerrada/Abonada";
        private const string FILTER_FECHA_CONTIENE = "Fecha contiene";
        
        private readonly string[] FILTERS = new string[] { FILTER_SIN_FILTRO,
                                                           FILTER_ESTADO_ABIERTA,
                                                           FILTER_ESTADO_ESPERANDO_AUTORIZACION,
                                                           FILTER_ESTADO_RECHAZADA,
                                                           FILTER_ESTADO_AUTORIZADA,
                                                           FILTER_ESTADO_CERRADA};

        private string _CurrentFilter;
        public string CurrentFilter { get { return _CurrentFilter; } set { _CurrentFilter = value; OnPropertyChanged(); OnCurrentFilterChanged(); } }

        private ObservableCollection<string> _FilterList = new ObservableCollection<string>();
        public ObservableCollection<string> FilterList { get { return _FilterList; } set { _FilterList = value; } }

        private string _FilterText = string.Empty;
        public string FilterText { get { return _FilterText; } set { _FilterText = value; OnPropertyChanged(); OnCurrentFilterChanged(); } }

        private void OnCurrentFilterChanged()
        {
            string newFilter = this.CurrentFilter;
            string filterText = this.FilterText;

            //Debug.Assert(newFilter == null, "El el texto del filtro tiene valor null");
            //Debug.Assert(filterText == null, "El filtro seleccionado tiene valor null");

            if (newFilter == null || filterText == null)
                return;

            ICollection<Liquidacion> liquidaciones = null;

            switch (newFilter)
            {
                case FILTER_SIN_FILTRO:
                    if (this.IsAuthorizationMode)
                        liquidaciones = this.LiquidacionManagement.ListLiquidacionesByAutorizador(this.CurrentUsuario,
                            EstadoLiquidacionType.Abierta,
                                                                                                  EstadoLiquidacionType.EsperandoAutorizacion,
                                                                                                  EstadoLiquidacionType.Rechazada,
                                                                                                  EstadoLiquidacionType.Autorizada,
                                                                                                  EstadoLiquidacionType.Pagada,
                                                                                                  EstadoLiquidacionType.Cerrada);
                    else
                        liquidaciones = this.LiquidacionManagement.ListLiquidacionesByUsuario(this.CurrentUsuario,
                                                                                              EstadoLiquidacionType.Abierta,
                                                                                              EstadoLiquidacionType.EsperandoAutorizacion,
                                                                                              EstadoLiquidacionType.Rechazada,
                                                                                              EstadoLiquidacionType.Autorizada,
                                                                                              EstadoLiquidacionType.Pagada,
                                                                                              EstadoLiquidacionType.Cerrada);
                    break;

                case FILTER_ESTADO_ABIERTA:
                    liquidaciones = this.LiquidacionManagement.ListLiquidacionesByUsuario(this.CurrentUsuario, EstadoLiquidacionType.Abierta);
                    break;

                case FILTER_ESTADO_ESPERANDO_AUTORIZACION:
                    if (this.IsAuthorizationMode)
                        liquidaciones = this.LiquidacionManagement.ListLiquidacionesByAutorizador(this.CurrentUsuario, EstadoLiquidacionType.EsperandoAutorizacion);
                    else
                        liquidaciones = this.LiquidacionManagement.ListLiquidacionesByUsuario(this.CurrentUsuario, EstadoLiquidacionType.EsperandoAutorizacion);
                    break;

                case FILTER_ESTADO_RECHAZADA:
                    if (this.IsAuthorizationMode)
                        liquidaciones = this.LiquidacionManagement.ListLiquidacionesByAutorizador(this.CurrentUsuario, EstadoLiquidacionType.Rechazada);
                    else
                        liquidaciones = this.LiquidacionManagement.ListLiquidacionesByUsuario(this.CurrentUsuario, EstadoLiquidacionType.Rechazada);
                    break;

                case FILTER_ESTADO_AUTORIZADA:
                    if (this.IsAuthorizationMode)
                        liquidaciones = this.LiquidacionManagement.ListLiquidacionesByAutorizador(this.CurrentUsuario, EstadoLiquidacionType.Autorizada);
                    else
                        liquidaciones = this.LiquidacionManagement.ListLiquidacionesByUsuario(this.CurrentUsuario, EstadoLiquidacionType.Autorizada);
                    break;

                case FILTER_ESTADO_CERRADA:
                    if (this.IsAuthorizationMode)
                        liquidaciones = this.LiquidacionManagement.ListLiquidacionesByAutorizador(this.CurrentUsuario, EstadoLiquidacionType.Cerrada);
                    else
                        liquidaciones = this.LiquidacionManagement.ListLiquidacionesByUsuario(this.CurrentUsuario, EstadoLiquidacionType.Cerrada);
                    break;
            }

            this.LiquidacionList.ReplaceRange(liquidaciones);
        }

        #endregion [Subsistema de filtrado]

    }
}

