﻿using LiquidacionGastos.Business;
using LiquidacionGastos.Framework;
using LiquidacionGastos.Models;
using LiquidacionGastos.Services;
using System;
using System.Collections.Generic;
using System.Diagnostics;
//using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace LiquidacionGastos.ViewModels
{
    public class LiquidacionViewModel : ViewModel
    {
        #region [Subsistema básico]

        /// <summary>
        /// Indica a la vista si debe mostrar la liquidaciones para el usuario que consulta ó para el autorizador.
        /// </summary>
        public bool IsAuthorizationMode { get; set; }

        public RelayCommand CancelCommand { get; private set; }

        public LiquidacionViewModel(bool isAuthorizationMode)
        {
            this.ViewId = ViewName.LiquidacionView;
            this.IsAuthorizationMode = isAuthorizationMode;

            if (this.IsAuthorizationMode)
            {
                this.ViewTitle = "Autorización de gastos";
                this.EditGastoCommandTitle = "Consultar gasto";
            }
            else
            {
                this.ViewTitle = "Liquidaciones de gastos";
                this.EditGastoCommandTitle = "Modificar gasto";
            }

            // Cargar los filtros de selección de gastos.
            this.FilterList.AddRange(this.FILTERS);

            // Seleccionar filtro sin efecto. 
            //this.CurrentFilter = FILTER_SIN_FILTRO;

            this.CancelCommand = new RelayCommand(OnCancel);
            this.CreateGastoCommand = new RelayCommand(OnCreateGasto, OnCanCreateGasto);
            this.EditGastoCommand = new RelayCommand(OnEditGasto, OnCanEditGasto);
            this.DeleteGastoCommand = new RelayCommand(OnDeleteGasto, OnCanDeleteGasto);

            this.SolicitarAutorizacionCommand = new RelayCommand(OnSolicitarAutorizacion, OnCanSolicitarAutorizacion);
            this.AutorizarCommand = new RelayCommand(OnAutorizar, OnCanAutorizar);
            this.RechazarCommand = new RelayCommand(OnRechazar, OnCanRechazar);
        }

        private void OnCancel(object param)
        {
            this.Stop(0);
        }

        #endregion [Subsistema básico]

        #region [Subsistema de mantenimiento]

        private Liquidacion _CurrentLiquidacion;
        public Liquidacion CurrentLiquidacion 
        {
            get
            {
                return _CurrentLiquidacion;
            }
            set
            {
                _CurrentLiquidacion = value;

                this.CurrentFilter = FILTER_SIN_FILTRO;

                OnPropertyChanged();
            }
        }

        public string NombreUsuario { get { return this._CurrentLiquidacion != null ? this._CurrentLiquidacion.Usuario.Nombre : string.Empty; } }
        public string FechaLiquidacion { get { return this._CurrentLiquidacion != null ? this._CurrentLiquidacion.Fecha.ToString("MM/yyyy") : string.Empty; } }
        public string EstadoLiquidacion { get { return this._CurrentLiquidacion != null ? this._CurrentLiquidacion.Estado.Nombre : string.Empty; } }
        public string MotivoRechazo { get { return this._CurrentLiquidacion != null ? this._CurrentLiquidacion.MotivoRechazo : string.Empty; } set { this._CurrentLiquidacion.MotivoRechazo = value; OnPropertyChanged(); } }
        public string EditGastoCommandTitle { get; set; }

        private Gasto _CurrentGasto;
        public Gasto CurrentGasto { get { return _CurrentGasto; } set { _CurrentGasto = value; OnPropertyChanged(); } }
        
        private ObservableCollection<Gasto> _GastoList = new ObservableCollection<Gasto>();
        public ObservableCollection<Gasto> GastoList { get { return _GastoList; } private set { _GastoList = value; OnPropertyChanged(); } }

        private LiquidacionManagement _GastoManagement = new LiquidacionManagement();
        private LiquidacionManagement GastoManagement { get { return _GastoManagement; } set { _GastoManagement = value; OnPropertyChanged(); } }
        
        public RelayCommand CreateGastoCommand { get; private set; }
        public RelayCommand EditGastoCommand { get; private set; }
        public RelayCommand DeleteGastoCommand { get; private set; }

        private bool OnCanCreateGasto(object obj)
        {
            return !this.IsAuthorizationMode;
        }

        private void OnCreateGasto(object param)
        {
            this.CurrentGasto = this.GastoManagement.GetNewGasto(this.CurrentLiquidacion);
            GastoViewModel vm = new GastoViewModel();
            vm.CurrentGasto = this.CurrentGasto;
            vm.Start(OnCreateGastoCallback);
        }

        private void OnCreateGastoCallback(ViewModel sender, ViewModelStoppedEventArgs e)
        {
            GastoViewModel vm = sender as GastoViewModel;
            if (e.Result != 1)
            {
                Debug.WriteLine("Datos del nuevo gasto descartados.");
                return;
            }

            this.GastoManagement.InsertGasto(vm.CurrentGasto);
            this.GastoList.Add(vm.CurrentGasto);
            Debug.WriteLine("Datos del nuevo gasto actualizados en la base de datos");

            // Actualizar el importe total de la liquidacion.
            this.GastoManagement.UpdateImporte(this.CurrentLiquidacion);
        }

        private bool OnCanEditGasto(object param)
        {
            return (this.CurrentGasto != null);
        }

        private void OnEditGasto(object param)
        {
            GastoViewModel vm = new GastoViewModel();
            vm.CurrentGasto = this.CurrentGasto;
            vm.Start(OnEditGastoCallback);
        }

        private void OnEditGastoCallback(ViewModel sender, ViewModelStoppedEventArgs e)
        {
            GastoViewModel vm = sender as GastoViewModel;
            if (e.Result != 1)
            {
                this.GastoManagement.RefreshGasto(vm.CurrentGasto);
                Debug.WriteLine("Datos del gasto recargados desde el contexto de la base de datos");
                return;
            }

            this.GastoManagement.UpdateGasto(vm.CurrentGasto);
            Debug.WriteLine("Datos del gasto actualizados en la base de datos");

            // Actualizar el importe total de la liquidacion.
            this.GastoManagement.UpdateImporte(this.CurrentLiquidacion);
        }

        private bool OnCanDeleteGasto(object obj)
        {
            return (this.CurrentGasto != null && !this.IsAuthorizationMode);
        }

        private void OnDeleteGasto(object obj)
        {
            DeleteGastoMessageViewModel vm = new DeleteGastoMessageViewModel();
            vm.Start(OnDeleteGastoCallback);
        }

        private void OnDeleteGastoCallback(ViewModel sender, ViewModelStoppedEventArgs e)
        {
            if (e.Result != 1)
                return;

            // Obtener el gasto seleccionado actualmente.
            Gasto gastoActual = this.CurrentGasto;

            // Eliminar el gasto de la lista. La propiedad this.CurrentGasto se convierte a null.
            this.GastoList.Remove(gastoActual);

            this.CurrentLiquidacion.Gastos.Remove(gastoActual);

            // Eliminar el gasto del sistema.
            this.GastoManagement.DeleteGasto(gastoActual);

            // Actualizar el importe total de la liquidacion.
            this.GastoManagement.UpdateImporte(this.CurrentLiquidacion);
        }

        #endregion [Subsistema de mantenimiento]

        #region [Subsistema de filtrado]

        private const string FILTER_SIN_FILTRO = "(sin filtro)";
        //private const string FILTER_NOMBRE_USUARIO = "nombre usuario";
        //private const string FILTER_NOMBRE_DEPARTAMENTO = "nombre departamento";

        private readonly string[] FILTERS = new string[] { FILTER_SIN_FILTRO };

        private string _CurrentFilter;
        public string CurrentFilter { get { return _CurrentFilter; } set { _CurrentFilter = value; OnPropertyChanged(); OnCurrentFilterChanged(); } }

        private ObservableCollection<string> _FilterList = new ObservableCollection<string>();
        public ObservableCollection<string> FilterList { get { return _FilterList; } set { _FilterList = value; } }

        private string _FilterText = string.Empty;
        public string FilterText { get { return _FilterText; } set { _FilterText = value; OnPropertyChanged(); OnCurrentFilterChanged(); } }

        private void OnCurrentFilterChanged()
        {
            string newFilter = this.CurrentFilter;
            string filterText = this.FilterText;

            //Debug.Assert(newFilter == null, "El el texto del filtro tiene valor null");
            //Debug.Assert(filterText == null, "El filtro seleccionado tiene valor null");

            if (this.CurrentLiquidacion == null || newFilter == null || filterText == null)
                return;

            ICollection<Gasto> gastos = null;

            switch (newFilter)
            {
                case FILTER_SIN_FILTRO:
                    gastos = this.GastoManagement.ListGastosByLiquidacion(this.CurrentLiquidacion);
                    break;

                //case FILTER_NOMBRE_USUARIO:
                //    usuarios = this.UsuarioManagement.ListUsuariosByNombreUsuario(filterText);
                //    break;

                //case FILTER_NOMBRE_DEPARTAMENTO:
                //    usuarios = this.UsuarioManagement.ListUsuariosByNombreDepartamento(filterText);
                //    break;
            }

            this.GastoList.ReplaceRange(gastos);
        }

        #endregion [Subsistema de filtrado]

        #region [Subsistema de Autorizacion]
        
        public bool IsReadOnlyMotivoRechazo { get { return !OnCanAutorizar(null); } }

        public RelayCommand SolicitarAutorizacionCommand { get; set; }
        public RelayCommand AutorizarCommand { get; set; }
        public RelayCommand RechazarCommand { get; set; }

        private bool OnCanRechazar(object obj)
        {
            return (this.IsAuthorizationMode &&
                    !string.IsNullOrEmpty(this.MotivoRechazo) &&
                    this.CurrentLiquidacion.Estado.IdEstadoLiquidacion == (int)EstadoLiquidacionType.EsperandoAutorizacion);
        }

        private void OnRechazar(object obj)
        {
            this.GastoManagement.ChangeEstadoLiquidacion(this.CurrentLiquidacion, EstadoLiquidacionType.Rechazada, this.MotivoRechazo);
            ToastNotification.Show("La liquidación se encuentra rechazada");
        }

        private bool OnCanAutorizar(object obj)
        {
            return (this.IsAuthorizationMode &&
                    this.CurrentLiquidacion.Estado.IdEstadoLiquidacion == (int)EstadoLiquidacionType.EsperandoAutorizacion);
        }

        private void OnAutorizar(object obj)
        {
            this.MotivoRechazo = string.Empty;
            this.GastoManagement.ChangeEstadoLiquidacion(this.CurrentLiquidacion, EstadoLiquidacionType.Autorizada);
            ToastNotification.Show("La liquidación se encuentra autorizada");
        }

        private bool OnCanSolicitarAutorizacion(object obj)
        {
            return (!this.IsAuthorizationMode && 
                    this.CurrentLiquidacion.Estado.IdEstadoLiquidacion != (int)EstadoLiquidacionType.EsperandoAutorizacion &&
                    this.CurrentLiquidacion.Estado.IdEstadoLiquidacion != (int)EstadoLiquidacionType.Pagada &&
                    this.CurrentLiquidacion.Estado.IdEstadoLiquidacion != (int)EstadoLiquidacionType.Cerrada);
        }

        private void OnSolicitarAutorizacion(object obj)
        {
            this.GastoManagement.ChangeEstadoLiquidacion(this.CurrentLiquidacion, EstadoLiquidacionType.EsperandoAutorizacion);
            ToastNotification.Show("La liquidación se encuentra a la espera de autorización");
        }

        #endregion
    }
}
