﻿using LiquidacionGastos.Business;
using LiquidacionGastos.Framework;
using LiquidacionGastos.Models;
using LiquidacionGastos.Services;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;

namespace LiquidacionGastos.ViewModels
{
    public class UsuarioListViewModel : ViewModel
    {
        #region [Subsistema básico]

        public RelayCommand CancelCommand { get; private set; }

        public UsuarioListViewModel(bool isSelectionMode)
        {
            this.ViewId = ViewName.UsuarioListView;
            this.UsuarioManagement = new OrganizacionManagement();
            this.UsuarioList = new ObservableCollection<Usuario>();
            this.IsSelectionMode = isSelectionMode;

            if (this.IsSelectionMode)
            {
                this.ViewTitle = "Selección de usuario";
                this.SelectionButtonVisibility = Visibility.Visible;
            }
            else
            {
                this.ViewTitle = "Lista de usuarios";
                this.SelectionButtonVisibility = Visibility.Hidden;
            }
            
            // Cargar los filtros de selección de usuarios.
            this.FilterList.AddRange(this.FILTERS);

            // Seleccionar filtro sin efecto. Se cargan todos los usuarios.
            this.CurrentFilter = FILTER_SIN_FILTRO;

            this.CancelCommand = new RelayCommand(OnCancel);
            this.CreateUsuarioCommand = new RelayCommand(OnCreateUsuario);
            this.EditUsuarioCommand = new RelayCommand(OnEditUsuario, OnCanEditUsuario);
            this.DeleteUsuarioCommand = new RelayCommand(OnDeleteUsuario, OnCanDeleteUsuario);
            this.SelectUsuarioCommand = new RelayCommand(OnSelectUsuario, OnCanSelectUsuario);
        }

        private void OnCancel(object param)
        {
            this.Stop(0);
        }

        #endregion [Subsistema básico]

        #region [Subsistema de mantenimiento]

        private Usuario _CurrentUsuario;
        public Usuario CurrentUsuario { get { return _CurrentUsuario; } set { _CurrentUsuario = value; OnPropertyChanged(); } }

        public ObservableCollection<Usuario> UsuarioList { get; private set; }

        private OrganizacionManagement UsuarioManagement { get; set; }

        public RelayCommand CreateUsuarioCommand { get; private set; }
        public RelayCommand EditUsuarioCommand { get; set; }
        public RelayCommand DeleteUsuarioCommand { get; private set; }

        private void OnCreateUsuario(object param)
        {
            this.CurrentUsuario = this.UsuarioManagement.GetNewUsuario();
            UsuarioViewModel vm = new UsuarioViewModel();
            vm.CurrentUsuario = this.CurrentUsuario;
            vm.Start(OnCreateUsuarioCallback);
        }

        private void OnCreateUsuarioCallback(ViewModel sender, ViewModelStoppedEventArgs e)
        {
            UsuarioViewModel vm = sender as UsuarioViewModel;
            if (e.Result != 1)
            {
                Debug.WriteLine("Datos del nuevo usuario descartados.");
                return;
            }

            this.UsuarioManagement.InsertUsuario(vm.CurrentUsuario);
            this.UsuarioList.Add(vm.CurrentUsuario);
            Debug.WriteLine("Datos del nuevo usuario actualizados en la base de datos");
        }

        private bool OnCanEditUsuario(object param)
        {
            return (this.CurrentUsuario != null);
        }

        private void OnEditUsuario(object param)
        {
            UsuarioViewModel vm = new UsuarioViewModel();
            vm.CurrentUsuario = this.CurrentUsuario;
            vm.Start(OnEditUsuarioCallback);
        }

        private void OnEditUsuarioCallback(ViewModel sender, ViewModelStoppedEventArgs e)
        {
            UsuarioViewModel vm = sender as UsuarioViewModel;
            if (e.Result != 1)
            {
                this.UsuarioManagement.RefreshUsuario(vm.CurrentUsuario);
                Debug.WriteLine("Datos del usuario recargados desde el contexto de la base de datos");
                return;
            }

            this.UsuarioManagement.UpdateUsuario(vm.CurrentUsuario);
            Debug.WriteLine("Datos del usuario actualizados en la base de datos");
        }

        private bool OnCanDeleteUsuario(object obj)
        {
            return (this.CurrentUsuario != null);
        }

        private void OnDeleteUsuario(object obj)
        {
            DeleteUsuarioMessageViewModel vm = new DeleteUsuarioMessageViewModel();
            vm.Start(OnDeleteUsuarioCallback);
        }

        private void OnDeleteUsuarioCallback(ViewModel sender, ViewModelStoppedEventArgs e)
        {
            if (e.Result != 1)
                return;

            // Obtener el usuario seleccionado actualmente.
            Usuario usuarioActual = this.CurrentUsuario;

            // Eliminar el usuario de la lista. La propiedad this.CurrentUsuario se convierte a null.
            this.UsuarioList.Remove(this.CurrentUsuario);

            // Eliminar el usuario del sistema.
            this.UsuarioManagement.DeleteUsuario(usuarioActual);
        }

        #endregion [Subsistema de mantenimiento]

        #region [Subsistema de filtrado]

        private const string FILTER_SIN_FILTRO = "(sin filtro)";
        private const string FILTER_NOMBRE_USUARIO = "nombre usuario";
        private const string FILTER_NOMBRE_DEPARTAMENTO = "nombre departamento";

        private readonly string[] FILTERS = new string[] { FILTER_SIN_FILTRO, FILTER_NOMBRE_USUARIO, FILTER_NOMBRE_DEPARTAMENTO };

        private string _CurrentFilter;
        public string CurrentFilter { get { return _CurrentFilter; } set { _CurrentFilter = value; OnPropertyChanged(); OnCurrentFilterChanged(); } }

        private ObservableCollection<string> _FilterList = new ObservableCollection<string>();
        public ObservableCollection<string> FilterList { get { return _FilterList; } set { _FilterList = value; } }

        private string _FilterText = string.Empty;
        public string FilterText { get { return _FilterText; } set { _FilterText = value; OnPropertyChanged(); OnCurrentFilterChanged(); } }

        private void OnCurrentFilterChanged()
        {
            string newFilter = this.CurrentFilter;
            string filterText = this.FilterText;

            //Debug.Assert(newFilter == null, "El el texto del filtro tiene valor null");
            //Debug.Assert(filterText == null, "El filtro seleccionado tiene valor null");

            if (newFilter == null || filterText == null)
                return;

            ICollection<Usuario> usuarios = null;

            switch (newFilter)
            {
                case FILTER_SIN_FILTRO:
                    usuarios = this.UsuarioManagement.ListUsuarios();
                    break;

                case FILTER_NOMBRE_USUARIO:
                    usuarios = this.UsuarioManagement.ListUsuariosByNombreUsuario(filterText);
                    break;

                case FILTER_NOMBRE_DEPARTAMENTO:
                    usuarios = this.UsuarioManagement.ListUsuariosByNombreDepartamento(filterText);
                    break;
            }

            this.UsuarioList.ReplaceRange(usuarios);
        }

        #endregion [Subsistema de filtrado]

        #region [Subsistema de selección]

        public bool IsSelectionMode { get; private set; }

        public Visibility SelectionButtonVisibility { get; private set; }

        public RelayCommand SelectUsuarioCommand { get; private set; }

        private bool OnCanSelectUsuario(object obj)
        {
            return (this.CurrentUsuario != null);
        }

        private void OnSelectUsuario(object obj)
        {
            Stop(1);
        }

        #endregion [Subsistema de selección]    
    }

}
