﻿using LiquidacionGastos.Business;
using LiquidacionGastos.Framework;
using LiquidacionGastos.Models;
using LiquidacionGastos.Services;
using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace LiquidacionGastos.ViewModels
{
    public class UsuarioViewModel : ViewModel
    {
        #region [Subsistema básico]

        // Cadenas constantes de los nombres de las propiedades que se validan.
        private const string PROPERTY_NAME_IDENTIFICADOR = "Identificador";
        private const string PROPERTY_NAME_PASSWORD = "Password";
        private const string PROPERTY_NAME_PASSWORD_RETYPED = "PasswordRetyped";
        private const string PROPERTY_NAME_NOMBRE = "Nombre";
        private const string PROPERTY_NAME_EMAIL = "Email";

        // Usuario actualmente en edición.
        private Usuario _CurrentUsuario;
        public Usuario CurrentUsuario { get { return _CurrentUsuario; } set { _CurrentUsuario = value; OnPropertyChanged(); } }

        // Propiedades del usuario.
        public string Identificador { get { return this.CurrentUsuario.Identificador; } set { this.CurrentUsuario.Identificador = value; OnPropertyChanged(); } }
        public string Email { get { return this.CurrentUsuario.Email; } set { this.CurrentUsuario.Email = value; OnPropertyChanged(); } }
        public bool EstaActivadoNotificarPorEmail { get { return this.CurrentUsuario.EstaActivadoNotificarPorEmail; } set { this.CurrentUsuario.EstaActivadoNotificarPorEmail = value; OnPropertyChanged(); } }
        public bool EstaActivadoLogin { get { return this.CurrentUsuario.EstaActivadoLogin; } set { this.CurrentUsuario.EstaActivadoLogin = value; OnPropertyChanged(); } }
        public string Nombre { get { return this.CurrentUsuario.Nombre; } set { this.CurrentUsuario.Nombre = value; OnPropertyChanged(); } }
                
        // Propiedades del ViewModel para el control de la constraseña.
        private string _Password;
        public string Password { get { return _Password; } set { _Password = value; OnPropertyChanged(); } }        
                      
        private string _PasswordRetyped;
        public string PasswordRetyped { get { return _PasswordRetyped; } set { _PasswordRetyped = value; OnPropertyChanged(); } }

        // Propiedades del ViewModel para asignar los textos de validación
        private string _IdentificadorValidationText;
        public string IdentificadorValidationText { get { return _IdentificadorValidationText; } set { _IdentificadorValidationText = value; OnPropertyChanged(); } }

        private string _PasswordValidationText;
        public string PasswordValidationText { get { return _PasswordValidationText; } set { _PasswordValidationText = value; OnPropertyChanged(); } }

        private string _NombreValidationText;
        public string NombreValidationText { get { return _NombreValidationText; } set { _NombreValidationText = value; OnPropertyChanged(); } }
                
        private string _EmailValidationText;
        public string EmailValidationText { get { return _EmailValidationText; } set { _EmailValidationText = value; OnPropertyChanged(); } }
                        
        // Comandos
        public RelayCommand CancelCommand { get; private set; }
        public RelayCommand SaveCommand { get; private set; }
        public RelayCommand SelectDepartmentCommand { get; private set; }

        public UsuarioViewModel()
        {
            this.ViewId = ViewName.UsuarioView;            
            this.Password = string.Empty;
            this.PasswordRetyped = string.Empty;
            this.CancelCommand = new RelayCommand(OnCancel);
            this.SaveCommand = new RelayCommand(OnSave, CanSave);
            this.SelectDepartmentCommand = new RelayCommand(OnSelectDepartamento);

            // Asignación de los campos de texto de validación para cada propiedad validada.
            this.AddValidationStringProperty("Identificador", "IdentificadorValidationText");
            this.AddValidationStringProperty("Password", "PasswordValidationText");
            this.AddValidationStringProperty("PasswordRetyped", "PasswordValidationText");
            this.AddValidationStringProperty("Nombre", "NombreValidationText");
            this.AddValidationStringProperty("Email", "EmailValidationText");            
        }

        private void OnCancel(object param)
        {
            this.Stop(0);
        }

        private bool CanSave(object obj)
        {
            return !this.HasErrors;
        }

        private void OnSave(object param)
        {
            if (this.Password.Length > 0 || this.PasswordRetyped.Length > 0)
            {
                if (this.Password.CompareTo(this.PasswordRetyped) == 0)
                {
                    OrganizacionManagement.SetPassword(this.CurrentUsuario, Password);
                }
            }
            this.Stop(1);
        }

        private void OnSelectDepartamento(object param)
        {
            DepartamentoListViewModel vm = new DepartamentoListViewModel(true);
            vm.CurrentDepartamento = this.CurrentUsuario.Departamento;
            vm.Start(OnSelectDepartamentoCallback);
        }

        private void OnSelectDepartamentoCallback(ViewModel sender, ViewModelStoppedEventArgs e)
        {
            DepartamentoListViewModel vm = sender as DepartamentoListViewModel;
            if (e.Result != 1)
                return;
    
            this.CurrentUsuario.Departamento = vm.CurrentDepartamento;
            this.CurrentUsuario.IdDepartamento = vm.CurrentDepartamento.IdDepartamento;
        }

        protected override void OnValidate(string propertyName)
        {
            if (string.IsNullOrEmpty(propertyName) || propertyName == PROPERTY_NAME_IDENTIFICADOR)
                OnValidateIdentificadorProperty();

            if (string.IsNullOrEmpty(propertyName) || propertyName == PROPERTY_NAME_NOMBRE)
                OnValidateNombreProperty();

            if (string.IsNullOrEmpty(propertyName) || propertyName == PROPERTY_NAME_EMAIL)
                OnValidateEmailProperty();

            if (string.IsNullOrEmpty(propertyName) || propertyName == PROPERTY_NAME_PASSWORD || propertyName == PROPERTY_NAME_PASSWORD_RETYPED)
                OnValidatePasswordProperty();
        }

        private void OnValidateIdentificadorProperty()
        {
            this.ClearValidationErrorMessages(PROPERTY_NAME_IDENTIFICADOR);
            if (string.IsNullOrEmpty(this.Identificador))
                this.AddValidationErrorMessage(PROPERTY_NAME_IDENTIFICADOR, "* Debe establecer un identificador para el proceso de login del usuario.");
        }

        private void OnValidateNombreProperty()
        {
            this.ClearValidationErrorMessages(PROPERTY_NAME_NOMBRE);
            if (string.IsNullOrEmpty(this.Nombre))
                this.AddValidationErrorMessage(PROPERTY_NAME_NOMBRE, "* Debe establecer el nombre del usuario.");
        }

        private void OnValidatePasswordProperty()
        {
            this.ClearValidationErrorMessages(PROPERTY_NAME_PASSWORD);
            if (this.Password.Length > 0 || this.PasswordRetyped.Length > 0)
                if (this.Password.CompareTo(this.PasswordRetyped) != 0)
                    this.AddValidationErrorMessage(PROPERTY_NAME_PASSWORD, "Las contraseñas no son coincidentes");
        }

        private void OnValidateEmailProperty()
        {
            this.ClearValidationErrorMessages(PROPERTY_NAME_EMAIL);
            if (this.EstaActivadoNotificarPorEmail && string.IsNullOrEmpty(this.Email))
            {
                this.AddValidationErrorMessage(PROPERTY_NAME_EMAIL, "Debe establecer un email para el usuario si activa la notificación.");
                return;
            }

            if (this.EstaActivadoNotificarPorEmail && !this.Email.Contains("@"))
            {
                this.AddValidationErrorMessage(PROPERTY_NAME_EMAIL, "Debe establecer un email con la forma: usuario@host.dominio");
            }
        }

        #endregion [Subsistema básico]
    }

}
