﻿using LiquidacionGastos.Framework;
using LiquidacionGastos.Services;
using System;
using System.Collections;
using System.Collections.Concurrent;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using System.Diagnostics;
using System.Linq;
using System.Linq.Expressions;
using System.Reflection;
using System.Runtime.CompilerServices;
using System.Text;
using System.Threading.Tasks;

namespace LiquidacionGastos.ViewModels
{
    public class ViewModel : ObservableObject, IViewModel//, INotifyDataErrorInfo
    {
        public delegate void ViewModelStoppedCallback(ViewModel sender, ViewModelStoppedEventArgs e);

        private ViewModelStoppedCallback StoppedCallback { get; set; }

        private bool _IsCreated;
        private bool _IsValidationEnabled;

        protected ViewName ViewId { get; set; }
        
        public string ViewTitle { get; set; }
        public event EventHandler Stopped;
        

        public int Result { get; protected set; }
        
        protected ViewModel()
        {
#if DEBUG
            string msg = string.Format("{0} ({1}) Created", this.GetType().Name, this.GetHashCode());
            System.Diagnostics.Debug.WriteLine(msg);
#endif
            _IsCreated = false;
            _IsValidationEnabled = false;
        }


        protected virtual void OnRegisterView()
        {
            ApplicationServices.NavigationService.RegisterView(this, this.ViewId);
        }

        protected virtual void OnStart()
        {
            ApplicationServices.NavigationService.Navigate(this, this.ViewId);
        }

        protected virtual void OnStop()
        {
            this.Dispose();
        }

        protected virtual void OnUnregisterView()
        {
            ApplicationServices.NavigationService.UnregisterView(this, this.ViewId);
        }

        private void OnStopped()
        {
            ViewModelStoppedCallback handler = this.StoppedCallback;

            if (handler != null)
            {
                ViewModelStoppedEventArgs e = new ViewModelStoppedEventArgs();
                e.Result = Result;
                this.StoppedCallback(this, e);
            }

            EventHandler handler2 = this.Stopped;

            if (handler2 != null)
                handler2(this, new EventArgs());
        }

        public void Start(ViewModelStoppedCallback callback)
        {
            this.StoppedCallback = callback;
            this.Start();
        }

        public void Start()
        {
            //this.StoppedCallback = null;

            if (!this._IsCreated)
            {            
                this.OnRegisterView();
                this._IsCreated = true;
            }

            this.OnStart();
            _IsValidationEnabled = true;
        }

        public void Stop(int result)
        {
            this.Result = result;
            if (result == 1)
            {
                this.Validate(null);

                if (this.HasErrors)
                    return;
            }
            
            this.Stop();
        }

        public void Stop()
        {
            this.OnStopped();
            this.OnStop();
        }

        #region [INotifyDataErrorInfo]

        private IDictionary<string, string> _ValidationErrorStringProperties = new Dictionary<string, string>();

        private void AddValidationStringProperty<T>(Expression<Func<T, string>> propertyExpr, Expression<Func<T, string>> validationStringPropertyExpr) where T : class
        {
            var expr = (MemberExpression)propertyExpr.Body;
            var prop = (PropertyInfo)expr.Member;

            var vsexpr = (MemberExpression)validationStringPropertyExpr.Body;
            var vsprop = (PropertyInfo)vsexpr.Member;
            this.AddValidationStringProperty(prop.Name, vsprop.Name);
        }

        private void RemoveValidationStringProperty(PropertyDescriptor property)
        {
            this.RemoveValidationStringProperty(property.Name);
        }

        protected void AddValidationStringProperty(string propertyName, string validationStringPropertyName)
        {
#if DEBUG
            // Verificar la existencia de todas las propiedades:

            PropertyDescriptor pd = TypeDescriptor.GetProperties(this)[propertyName];
            Debug.WriteLine(string.Format("No se puede acceder a la propiedad {0} porque no existe.", propertyName));
            Debug.Assert(pd != null);

            pd = TypeDescriptor.GetProperties(this)[validationStringPropertyName];
            Debug.WriteLine(string.Format("No se puede acceder a la propiedad {0} porque no existe.", validationStringPropertyName));
            Debug.Assert(pd != null);
#endif

            if (_ValidationErrorStringProperties.ContainsKey(propertyName))
                _ValidationErrorStringProperties[propertyName] = validationStringPropertyName;
            else
                _ValidationErrorStringProperties.Add(propertyName, validationStringPropertyName);
        }

        protected void RemoveValidationStringProperty(string propertyName)
        {
            if (_ValidationErrorStringProperties.ContainsKey(propertyName))
                _ValidationErrorStringProperties.Remove(propertyName);
        }

        private IDictionary<string, List<string>> _errors = new Dictionary<string, List<string>>();

        //public event EventHandler<DataErrorsChangedEventArgs> ErrorsChanged;

        public void OnErrorsChanged(string propertyName)
        {
            //var handler = ErrorsChanged;
            //if (handler != null)
            //    handler(this, new DataErrorsChangedEventArgs(propertyName));

            if (!_ValidationErrorStringProperties.ContainsKey(propertyName))
                return;

            string validationPropertyName = _ValidationErrorStringProperties[propertyName];

            if (string.IsNullOrEmpty(validationPropertyName))
                throw new ApplicationException("No se puede actualizar la propiedad del texto de validación porque no está registrada.");

            
            IEnumerable errorList = this.GetErrors(propertyName);
            
            string errorMessage = string.Empty;
            foreach(string error in errorList)
            {
                if (errorMessage.Length > 0)
                    errorMessage += "\n";

                errorMessage += error;
            }

            this.SetValue(validationPropertyName, errorMessage);
        }

        public IEnumerable GetErrors(string propertyName)
        {
            Debug.WriteLine(string.Format("-> GetErrors({0})", string.IsNullOrEmpty(propertyName) ? "(vacio)" : propertyName));

            List<string> errorDescriptions = null;

            if (_errors.ContainsKey(propertyName))
                errorDescriptions = _errors[propertyName];

            return errorDescriptions;
        }

        public bool HasErrors
        {
            get { return _errors.Any(kv => kv.Value != null && kv.Value.Count > 0); }
        }

        protected void Validate(string propertyName)
        {
            this.OnValidate(propertyName);
        }

        protected virtual void OnValidate(string propertyName)
        {
        }

        protected override void OnPropertyChanged(string propertyName = "")
        {
            base.OnPropertyChanged(propertyName);

            if (_IsValidationEnabled)
            {
                _IsValidationEnabled = false;
                this.Validate(propertyName);
                _IsValidationEnabled = true;
            }
        }


        private void ClearValidationErrorMessages(PropertyDescriptor property)
        {
            this.ClearValidationErrorMessages(property.Name);
        }

        protected void ClearValidationErrorMessages(string propertyName)
        {
            if (string.IsNullOrEmpty(propertyName))
            {
                this._errors.Clear();
            }

            if (!this._errors.ContainsKey(propertyName))
                return;

            this._errors[propertyName].Clear();
            OnErrorsChanged(propertyName);
        }

        private void AddValidationErrorMessage(PropertyDescriptor property, string message)
        {
            this.AddValidationErrorMessage(property.Name, message);
        }

        protected void AddValidationErrorMessage(string propertyName, string message)
        {
            List<string> errorMessageList = null;

            if (!_errors.ContainsKey(propertyName))
                _errors.Add(propertyName, new List<string>());

            errorMessageList = this._errors[propertyName];

            if (!errorMessageList.Contains(message))
            {
                errorMessageList.Add(message);
                OnErrorsChanged(propertyName);
            }
        }

        private void RemoveValidationErrorMessage(PropertyDescriptor property, string message)
        {
            this.RemoveValidationErrorMessage(property.Name, message);
        }

        protected void RemoveValidationErrorMessage(string propertyName, string message)
        {
            List<string> errorMessageList = null;

            if (!_errors.ContainsKey(propertyName))
                return;

            errorMessageList = _errors[propertyName];

            if (errorMessageList.Contains(message))
            {
                errorMessageList.Remove(message);
                OnErrorsChanged(propertyName);
            }
        }

        #endregion [INotifyDataErrorInfo]

        protected void SetValue(string propertyName, object value)
        {
            PropertyDescriptor pd = this.GetPropertyByName(propertyName);
            pd.SetValue(this, value);
        }

        protected object GetValue(string propertyName, object value)
        {
            PropertyDescriptor pd = this.GetPropertyByName(propertyName);
            return pd.GetValue(this);
        }

        private PropertyDescriptor GetPropertyByName(string propertyName)
        {
            if (string.IsNullOrEmpty(propertyName))
            {
                string s = string.Format("No se puede acceder a una sin especificar su nombre.");
                throw new ApplicationException(s);
            }

            PropertyDescriptor pd = TypeDescriptor.GetProperties(this)[propertyName];

            if (pd == null)
            {
                string s = string.Format("No se puede acceder a la propiedad {0} porque no existe.", propertyName);
                throw new ApplicationException(s);
            }

            return pd;
        }

        #region Debugging Aides

        /// <summary>
        /// Warns the developer if this object does not have
        /// a public property with the specified name. This 
        /// method does not exist in a Release build.
        /// </summary>
        [Conditional("DEBUG")]
        [DebuggerStepThrough]
        public void VerifyPropertyName(string propertyName)
        {
            // Verify that the property name matches a real,  
            // public, instance property on this object.
            if (TypeDescriptor.GetProperties(this)[propertyName] == null)
            {
                string msg = "Invalid property name: " + propertyName;

                if (this.ThrowOnInvalidPropertyName)
                    throw new Exception(msg);
                else
                    Debug.Fail(msg);
            }
        }

        /// <summary>
        /// Returns whether an exception is thrown, or if a Debug.Fail() is used
        /// when an invalid property name is passed to the VerifyPropertyName method.
        /// The default value is false, but subclasses used by unit tests might 
        /// override this property's getter to return true.
        /// </summary>
        protected virtual bool ThrowOnInvalidPropertyName { get; private set; }

        #endregion // Debugging Aides

        #region IDisposable Members

        /// <summary>
        /// Invoked when this object is being removed from the application
        /// and will be subject to garbage collection.
        /// </summary>
        public void Dispose()
        {
            this.OnDispose();
        }

        /// <summary>
        /// Child classes can override this method to perform 
        /// clean-up logic, such as removing event handlers.
        /// </summary>
        protected virtual void OnDispose()
        {
            this.OnUnregisterView();
        }

#if DEBUG
        /// <summary>
        /// Useful for ensuring that ViewModel objects are properly garbage collected.
        /// </summary>
        ~ViewModel()
        {
            string msg = string.Format("{0} ({1}) Finalized", this.GetType().Name, this.GetHashCode());
            System.Diagnostics.Debug.WriteLine(msg);
        }
#endif

        #endregion // IDisposable Members
    }
}
